@@ -16,15 +16,15 @@ if(identical(Sys.getenv("NOT_CRAN"), "true")& .Machine$sizeof.pointer != 4){
     par1<-rnorm(nsubjects,-.3,.8)
     par2 <- par1*.4 + rnorm(nsubjects,0,.3)
     for(i in 1:nsubjects){
-      gm<-ctModel(type='omx',n.latent=2,n.manifest=n.manifest,Tpoints=Tpoints,LAMBDA=matrix(c(1,.4,0,0,0,1),3,ncol=2),
+      gm<-suppressMessages(ctModel(type='omx',n.latent=2,n.manifest=n.manifest,Tpoints=Tpoints,LAMBDA=matrix(c(1,.4,0,0,0,1),3,ncol=2),
         DRIFT=diag(-.4,2),
         CINT=matrix(c(par1[i],par2[i]),2),
         T0VAR=diag(1,2),
         T0MEANS=matrix(c(30,50),n.latent),
         MANIFESTVAR=t(chol(diag(.5,n.manifest))),
-        DIFFUSION=t(chol(diag(3,2))))
-      if(i==1) cd<-ctGenerate(gm,n.subjects=1,burnin=burnin,wide=FALSE,dtmat = dtmat) else {
-        newdat <- ctGenerate(gm,n.subjects=1,burnin=burnin,wide=FALSE,dtmat = dtmat)
+        DIFFUSION=t(chol(diag(3,2)))))
+      if(i==1) cd<-suppressMessages(ctGenerate(gm,n.subjects=1,burnin=burnin,wide=FALSE,dtmat = dtmat)) else {
+        newdat <- suppressMessages(ctGenerate(gm,n.subjects=1,burnin=burnin,wide=FALSE,dtmat = dtmat))
         newdat[,'id'] <- i
         cd<-rbind(cd,newdat)
       }
@@ -59,28 +59,19 @@ if(identical(Sys.getenv("NOT_CRAN"), "true")& .Machine$sizeof.pointer != 4){
     sm1$pars$indvarying <- FALSE
 
     sf1 <- ctStanFit(cd,sm1,iter=200,
-      optimize=TRUE,verbose=1,nopriors = TRUE,
+      optimize=TRUE,verbose=0,
       optimcontrol=list(finishsamples=50),
-      derrind=1:4,
       nlcontrol=list(nldynamics=FALSE))
 
     
     sf1d=sf1$stanfit$transformedparsfull$pop_DRIFT[1,1:2,1:2]#matrix(sf1$stanfit$transformedpars_old[grep('pop_DRIFT',rownames(sf1$stanfit$transformedpars_old)),'mean'],4,4)[1:2,1:2]
     sf1ll=sf1$stanfit$optimfit$value
     
-    sf1_derrind<- ctStanFit(cd,sm1,iter=200,
-      optimize=TRUE,verbose=0,nopriors = TRUE,
-      optimcontrol=list(deoptim = FALSE,is=FALSE,isloopsize=50,finishsamples=50),
-      derrind=1:2,
-      nlcontrol=list(nldynamics=FALSE))
-    sf1_derrindd=sf1_derrind$stanfit$transformedparsfull$pop_DRIFT[1,1:2,1:2]#matrix(sf1_derrind$stanfit$transformedpars_old[grep('pop_DRIFT',rownames(sf1_derrind$stanfit$transformedpars_old)),'mean'],4,4)[1:2,1:2]
-    sf1_derrindll=sf1_derrind$stanfit$optimfit$value
-    
     sm2 <- ctStanModel(m2)
     sm2$pars$sdscale <- .2
     
     sf2 <- ctStanFit(cd,sm2,
-      optimize=T,verbose=0,nopriors = TRUE)
+      optimize=T,verbose=0)
 
     sf2d=sf2$stanfit$transformedparsfull$pop_DRIFT[1,1:2,1:2]# matrix(sf2$stanfit$transformedpars_old[grep('pop_DRIFT',rownames(sf2$stanfit$transformedpars_old)),'mean'],4,4)[1:2,1:2]
     sf2ll=sf2$stanfit$optimfit$value
@@ -102,8 +93,8 @@ if(identical(Sys.getenv("NOT_CRAN"), "true")& .Machine$sizeof.pointer != 4){
       }
     }
     
-    dvec=c('sf1d','sf2d','sf1_derrindd')
-    llvec=c('sf1ll','sf2ll','sf1_derrindll')
+    dvec=c('sf1d','sf2d')
+    llvec=c('sf1ll','sf2ll')
     
     sapply(dvec,get,envir=sys.frame(sys.parent(0)))
     sapply(llvec,get,envir=sys.frame(sys.parent(0)))
@@ -128,8 +119,8 @@ if(identical(Sys.getenv("NOT_CRAN"), "true")& .Machine$sizeof.pointer != 4){
     drift= -2*log1p(exp(2*(rnorm(nsubjects,driftmu,driftsd))))
     
     for(si in 1:nsubjects){
-      m=ctModel(LAMBDA=diag(1), Tpoints=Tpoints, DRIFT=matrix(drift[si]),T0MEANS = matrix(3), 
-        T0VAR=matrix(sqrt(.1)), DIFFUSION=diag(1,1), CINT=matrix(-2),MANIFESTVAR=matrix(sqrt(.1)))
+      m=suppressMessages(ctModel(LAMBDA=diag(1), Tpoints=Tpoints, DRIFT=matrix(drift[si]),T0MEANS = matrix(3), 
+        T0VAR=matrix(sqrt(.1)), DIFFUSION=diag(1,1), CINT=matrix(-2),MANIFESTVAR=matrix(sqrt(.1))))
       d=suppressMessages(ctGenerate(m,n.subjects = 1,burnin = 0,wide = FALSE,dtmean = dt))[,-1]
       if(si==1) dat=cbind(si,d) else dat=rbind(dat,cbind(si,d))
     }
@@ -146,7 +137,7 @@ if(identical(Sys.getenv("NOT_CRAN"), "true")& .Machine$sizeof.pointer != 4){
     dtm$pars$indvarying[dtm$pars$matrix %in% 'DRIFT'] <- TRUE
     
     dtf = ctStanFit(datalong = dat,ctstanmodel = dtm,optimize=TRUE,
-      verbose=0,optimcontrol=list(estonly=F),savescores = F,nopriors=T)
+      verbose=0,optimcontrol=list(estonly=F),savescores = F)
     s1=summary(dtf,parmatrices = F,priorcheck = F,residualcov = F)
     
     dtm2 <- ctModel(LAMBDA=matrix(c(1,0),1,2), type='stanct',
@@ -156,7 +147,7 @@ if(identical(Sys.getenv("NOT_CRAN"), "true")& .Machine$sizeof.pointer != 4){
       MANIFESTMEANS = matrix(0))
     
     dtm2$pars$indvarying <- FALSE
-    dtf2=ctStanFit(datalong = dat,ctstanmodel = dtm2,optimize = TRUE,nopriors = T)
+    dtf2=ctStanFit(datalong = dat,ctstanmodel = dtm2,optimize = TRUE)
     s2=summary(dtf2,parmatrices = F,priorcheck = F,residualcov = F)
     
     # expect_equivalent(c(s1$popmeans[,1],s1$popsd[,1]),s2$popmeans[,1],tol=1e-2)
